/*******************************************************************\

Module:

Author: Daniel Kroening, kroening@kroening.com

\*******************************************************************/

#include <libdbox/stringtools.h>
#include "base64str.h"
#include "httpd_base.h"
#include "str_getline.h"

/*******************************************************************\

Function: httpd_baset::parse_command

  Inputs:

 Outputs:

 Purpose:

\*******************************************************************/

void httpd_baset::parse_command(char *c)
 {
  char *t1, *t2, old;

  method=UNKNOWN;

  //
  // Method
  //

  t2=c;
  while(*t2!=' ' && *t2!='\t' && *t2!=0) t2++;
  old=*t2;
  *t2=0;
  
  if(stringcasecmp(command, "GET")==0)
    method=GET;
  else if(stringcasecmp(command, "PUT")==0)
    method=PUT;
  else if(stringcasecmp(command, "POST")==0)
    method=POST;

  *t2=old;
  while(*t2==' ' || *t2=='\t') t2++;
  t1=t2;

  if(t1[0]==0) 
   {
    url=version="";
    return;
   }

  // 
  // URL
  // 

  while(*t2!=' ' && *t2!='\t' && *t2!=0 && *t2!='?') t2++;
  old=*t2;
  *t2=0;
  url=t1;
  decode_url(url);
  *t2=old;

  if(old=='?')
   {
    t1=t2+1;
    while(*t2!=' ' && *t2!='\t' && *t2!=0) t2++;
    old=*t2;
    *t2=0;
    parse_parameter(t1);
    *t2=old;
   }

  while(*t2==' ' || *t2=='\t') t2++;
  t1=t2;

  if(t1[0]==0)
   {
    version="";
    return;
   }

  // 
  // Version
  //

  version=t1;
 }

/*******************************************************************\

Function: httpd::parse_command

  Inputs:

 Outputs:

 Purpose:

\*******************************************************************/

void httpd_baset::parse_command()
 {
  if(command=="")
   {
    method=UNKNOWN;
    url=version="";
    return;
   }

  char *c=strdup(command.c_str());

  parse_command(c);

  free(c);
 }

/*******************************************************************\

Function: httpd_baset::parse_parameter

  Inputs:

 Outputs:

 Purpose:

\*******************************************************************/

void httpd_baset::parse_parameter(char *str)
 {
  char *t;

  if(str==NULL) return;

  t=str;
  while(*t!=0)
   {
    if(*t==-1)
      *t=' ';

    t++;
   }

  while(str[0]!=0)
   {
    t=strchr(str, '=');
    if(t==NULL) return;
    *t=0;

    std::string _name=str;
    decode_url(_name);

    *t='=';
    str=t+1;

    std::string _inhalt;
    t=strchr(str, '&');
    if(t==NULL)
     {
      _inhalt=str;
      str="";
     }
    else
     {
      *t=0;
      _inhalt=str;
      *t='&';
      str=t+1;
     }

    decode_url(_inhalt);

    parameters[_name]=_inhalt;
   }
 }

/*******************************************************************\

Function: httpd_baset::httpd_baset

  Inputs:

 Outputs:

 Purpose:

\*******************************************************************/

httpd_baset::httpd_baset(std::istream &_in, std::ostream &_out):
  in(_in), out(_out)
 {
  method=UNKNOWN;
 }

/*******************************************************************\

Function: httpd_baset::basic_auth 

  Inputs:

 Outputs:

 Purpose:

\*******************************************************************/

void httpd_baset::basic_auth(std::string &username, std::string &password)
 {
  username=password="";

  std::string &header=headers["Authorization"];

  if(std::string(header, 0, 6)=="Basic ")
   {
    std::string buffer;
    base64_decode(std::string(header, 6), buffer);

    std::string::size_type pos=buffer.find(':');
    if(pos==std::string::npos) return;

    username=std::string(buffer, 0, pos-1);
    password=std::string(buffer, pos+1);
   }
 }

/*******************************************************************\

Function:

  Inputs:

 Outputs:

 Purpose:

\*******************************************************************/

httpd_baset::~httpd_baset()
 {
  //clear();
 }

/*******************************************************************\

Function: httpd_baset::decode_url

  Inputs:

 Outputs:

 Purpose:

\*******************************************************************/

void httpd_baset::decode_url(std::string &url)
 {
  char mime_data=0;
  int status=0;
  std::string buffer;

  buffer.swap(url);

  url="";

  for(unsigned i=0; i<buffer.size(); i++)
   {
    char data=buffer[i];

    switch(data)
     {
     case '\n':
     case '\r':
     case 0:
      return;

     case '%':
      status=1;
      break;
    
     case '+':
      url+=' ';
      break;

     default:
      switch(status)
       {
       case 0:
        url+=data;
        break;

       case 1:
        status++;
        if(isascii(data) && isxdigit(data))
         {
          if(isdigit(data))
            mime_data=(data-'0')<<4;
          else
            mime_data=(tolower(data)-'a'+10)<<4;
         }
        else if(data=='%')
          url+=data;
        break;

       case 2:
        status=0;
        if(isascii(data) && isxdigit(data))
         {
          if(isdigit(data))
            mime_data|=data-'0';
          else
            mime_data|=tolower(data)-'a'+10;
          
          url+=mime_data;
         }
        break;

       default:
        abort();
       }
     }
   }
 }

/*******************************************************************\

Function: httpd_baset::read_header

  Inputs:

 Outputs:

 Purpose:

\*******************************************************************/

void httpd_baset::read_header()
 {
  std::string buffer;

  while(TRUE)
   {
    str_getline(in, buffer);
    if(buffer=="") return;

    char *b=strdup(buffer.c_str());
    char *tptr=strchr(b, ':');

    if(tptr!=NULL)
     {
      *tptr=0;
      tptr++;
      while(*tptr==' ' || *tptr=='\t') tptr++;

      if(buffer[0]!=0)
        headers[b]=tptr;
     }

    free(b);
   }
 }

/*******************************************************************\

Function: httpd_baset::httpd_baset

  Inputs:

 Outputs:

 Purpose:

\*******************************************************************/

void httpd_baset::httpd()
 {
  read_command();
  parse_command();

  if(version!="")
    read_header();

  try
   {
    process_request();
   }

  catch(...)
   {
    // ignore it
   }
 }

/*******************************************************************\

Function: httpd_baset::read_command 

  Inputs:

 Outputs:

 Purpose:

\*******************************************************************/

void httpd_baset::read_command()
 {
  str_getline(in, command);
 }

/*******************************************************************\

Function: httpd_baset::process_request

  Inputs:

 Outputs:

 Purpose:

\*******************************************************************/

void httpd_baset::process_request()
 {
  html << "<html>\n"
          "<title>Error: unknown URL</title>\n"
          "<body>\n"
          "unknown URL: " << url << "\n"
          "</body>\n"
          "</html>\n";

  error(404, "Not Found");
 }

/*******************************************************************\

Function: httpd_baset::status_response

  Inputs:

 Outputs:

 Purpose:

\*******************************************************************/

void httpd_baset::status_response(unsigned code, const std::string &msg)
 {
  out << "HTTP/1.1 " << code << " " << msg << "\r\n";
 }

/*******************************************************************\

Function: httpd_baset::send_header

  Inputs:

 Outputs:

 Purpose:

\*******************************************************************/

void httpd_baset::send_header(const std::string &content_type,
                              unsigned long content_length)
 {
  out << "Date: " << "Tue, 18 Mar 2003 21:03:37 GMT" << "\r\n";
  send_server();
  send_connection();
  out << "Last-Modified: " << "Mon, 16 Dec 2002 21:51:31 GMT" << "\r\n"
      << "Content-Length: " << content_length << "\r\n"
      << "Content-Type: " << content_type << "\r\n"
      << "\r\n";
 }

/*******************************************************************\

Function: httpd_baset::send_connection

  Inputs:

 Outputs:

 Purpose:

\*******************************************************************/

void httpd_baset::send_connection()
 {
  out << "Connection: " << "close" << "\r\n";
 }

/*******************************************************************\

Function: httpd_baset::send_html

  Inputs:

 Outputs:

 Purpose:

\*******************************************************************/

void httpd_baset::send_html()
 {
  error(200, "OK");
 }

/*******************************************************************\

Function: httpd_baset::error

  Inputs:

 Outputs:

 Purpose:

\*******************************************************************/

void httpd_baset::error(unsigned code, const std::string &msg)
 {
  status_response(code, msg);
  send_header("text/html", html.str().size());
  out << html.str();
 }

