/*******************************************************************\

Module:

Author: Daniel Kroening, kroening@kroening.com

\*******************************************************************/

#include <dblib/str_getline.h>
#include <dblib/excpt.h>

#include "index.h"

/*******************************************************************\

Function: indext::open

  Inputs:

 Outputs:

 Purpose:

\*******************************************************************/

void indext::open(const std::string &filename)
 {
  data.open(filename.c_str());
  index.open((filename+".idx").c_str(), std::ios::in|std::ios::binary);

  if(!data)
    throw excpt("failed to open "+filename);

  if(!index)
    throw excpt("failed to open "+filename+".idx");
 }

/*******************************************************************\

Function: indext::find

  Inputs:

 Outputs:

 Purpose:

\*******************************************************************/

bool indext::find(const std::string &id, std::string &dest)
 {
  index.seekg(0, std::ios::end);
  std::streampos length=index.tellg();

  dest="";

  unsigned long low=0;
  unsigned long high=length/sizeof(std::streampos);

  while(true)
   {
    if(low>=high) return true; // not found

    unsigned long i=(low+high)/2;

    index.seekg(i*sizeof(std::streampos));
    std::streampos pos;
    if(!index.read((char *)&pos, sizeof(std::streampos)))
      throw excpt("failed to read index file");

    std::string line;
    data.seekg(pos);

    if(!str_getline(data, line))
      throw excpt("failed to read data file");

    std::string::size_type p=line.find('=');
    if(p==std::string::npos)
      throw excpt("data file format error");

    std::string _id(line, 0, p);
    if(id<_id)
      high=i;
    else if(id>_id)
      low=i+1;
    else // id==_id
     {
      dest=std::string(line, p+1);
      return false;
     }
   }
 }

