/*******************************************************************\

Module:

Author: Daniel Kroening, kroening@kroening.com

\*******************************************************************/

#include <dblib/trfalse.h>
#include <dblib/str_getline.h>
#include <dblib/stream_table.h>

/*******************************************************************\

Function: stream_tablet::stream_tablet

  Inputs:

 Outputs:

 Purpose:

\*******************************************************************/

stream_tablet::stream_tablet()
 {
 }

/*******************************************************************\

Function: stream_tablet::stream_tablet

  Inputs:

 Outputs:

 Purpose:

\*******************************************************************/

stream_tablet::stream_tablet(const std::string &__name):
  table_baset(__name)
 {
 }

/*******************************************************************\

Function: stream_tablet::~stream_tablet

  Inputs:

 Outputs:

 Purpose:

\*******************************************************************/

stream_tablet::~stream_tablet()
 {
 }

/*******************************************************************\

Function: stream_tablet::read_map_line

  Inputs:

 Outputs:

 Purpose:

\*******************************************************************/

bool stream_tablet::read_map_line(std::istream &in,
                                  std::string &id,
                                  std::string &value)
 {
  do
   {
    std::string line;

    if(in.eof() || in.fail())
      return TRUE;

    str_getline(in, line);

    id.erase();
    value.erase();

    unsigned i=0;

    // skip whitespace
    for(; (line[i]==' ' || line[i]=='\t') && i<line.size(); i++);

    for(; i<line.size(); i++)
      if(line[i]=='=')
       {
        i++;
        break;
       }
      else if(line[i]!='\r' && line[i]!='\n')
        id+=line[i];

    // skip whitespace
    for(; (line[i]==' ' || line[i]=='\t') && i<line.size(); i++);

    for(; i<line.size(); i++)
      if(line[i]!='\r' && line[i]!='\n')
        value+=line[i];
   }
  while(id=="" || id[0]==';' || id[0]=='#'); // skip comments

  return FALSE;
 }

/*******************************************************************\

Function: stream_tablet::read_line

  Inputs:

 Outputs:

 Purpose:

\*******************************************************************/

bool stream_tablet::read_line(std::istream &in, std::string &value)
 {
  std::string line;

  if(in.eof() || in.fail())
    return TRUE;

  str_getline(in, line);

  value.erase();

  for(unsigned i=0; i<line.size(); i++)
    if(line[i]!='\r' && line[i]!='\n')
      value+=line[i];

  return FALSE;
 }

/*******************************************************************\

Function: stream_tablet::read

  Inputs:

 Outputs:

 Purpose:

\*******************************************************************/

bool stream_tablet::read(std::istream &in)
 {
  switch(_type)
   {
   case NOT_SET:
    assert(FALSE);
    break;

   case MAP:
     {
      std::string id, value;

      get_map().clear();

      while(!read_map_line(in, id, value))
        get_map().insert(std::pair<std::string, std::string>(id, value));
     }
    break;

   case SET:
     {
      std::string value;

      get_set().clear();

      while(!read_line(in, value))
        get_set().insert(value);
     }
    break;
     
   case LIST:
     {
      std::string value;

      get_list().clear();

      while(!read_line(in, value))
        get_list().push_back(value);
     }
    break;

   case VALUE:
    if(read_line(in, get_value()))
      return TRUE;
    break;
   }

  return FALSE;
 }

/*******************************************************************\

Function: stream_tablet::write

  Inputs:

 Outputs:

 Purpose:

\*******************************************************************/

bool stream_tablet::write(std::ostream &out) const
 {
  switch(_type)
   {
   case NOT_SET:
    assert(FALSE);
    break;

   case MAP:
    for(mapt::const_iterator it=_map.begin(); it!=_map.end(); it++)
      out << it->first << "=" << it->second << "\n";
    break;

   case SET:
    for(sett::const_iterator it=_set.begin(); it!=_set.end(); it++)
      out << *it << "\n";
    break;
     
   case LIST:
    for(listt::const_iterator it=_list.begin(); it!=_list.end(); it++)
      out << *it << "\n";
    break;

   case VALUE:
    out << _value << "\n";
    break;
   }

  return FALSE;
 }
