/*******************************************************************\

Module:

Author: Daniel Kroening, kroening@kroening.com

\*******************************************************************/

#ifndef DBLIB_TABLE_BASE_H
#define DBLIB_TABLE_BASE_H

#include <stdlib.h>

#include <set>
#include <map>
#include <list>
#include <string>

#include <dblib/excpt.h>
#include <dblib/i2string.h>

EXCPT(table_excpt);

class table_baset
 {
 public:
  table_baset(const std::string &__name):_name(__name)
   { _type=NOT_SET; }
   
  table_baset()
   { _type=NOT_SET; }
   
  typedef std::list<std::string> listt;
  typedef std::map<std::string, std::string> mapt;
  typedef std::set<std::string> sett;
  typedef std::string valuet;
  
  inline listt &get_list() { return _list; }
  inline const listt &get_list() const { return _list; }

  inline sett &get_set() { return _set; }
  inline const sett &get_set() const { return _set; }

  inline mapt &get_map() { return _map; }
  inline const mapt &get_map() const { return _map; }
  
  inline const long get_int(const std::string &id) const
   {
    return atol((*this)[id].c_str());
   }
  
  inline const long get_bool(const std::string &id) const
   {
    return atol((*this)[id].c_str());
   }
  
  inline valuet &get_value() { return _value; }
  inline const valuet &get_value() const { return _value; }
  
  typedef enum { NOT_SET, LIST, SET, MAP, VALUE } typet;
 
  inline typet type() const { return _type; }

  inline void set_type(typet __type)
   { _type=__type; }
   
  inline void set_name(const std::string &__name)
   { _name=__name; }
   
  void set(const std::string &id, const std::string &value)
   {
    get_map()[id]=value;
   }
   
  void set(const std::string &id, unsigned int i)
   {
    set(id, i2string(i));
   }
   
  void set(const std::string &id, signed int i)
   {
    set(id, i2string(i));
   }
   
  void set(const std::string &id, unsigned long int i)
   {
    set(id, i2string(i));
   }
   
  void set(const std::string &id, signed long int i)
   {
    set(id, i2string(i));
   }
   
  const std::string &operator[](const std::string &id) const
   {
    mapt::const_iterator it=get_map().find(id);
    if(it==get_map().end()) return empty_string;
    return it->second;
   }
   
  void swap(table_baset &table)
   {
    std::swap(table._type, _type);
    _name.swap(table._name);
    _list.swap(table._list);
    _set.swap(table._set);
    _map.swap(table._map);
    _value.swap(table._value);
   }
 
 protected:
  std::string _name;
  typet _type;
  listt _list;
  sett _set;
  mapt _map;
  valuet _value;
  
  std::string empty_string;
 };

#endif
