/*

  DBOX Module
  Copyright (C) 1994-2000 Daniel Kroening <kroening@handshake.de>

  Purpose: 

*/

#include <stdio.h>
#include <time.h>
#include <unistd.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>

#include <iostream>
#include <string>
#include <list>

#include <stringtools.h>
#include <libdbox/dboxversion.h>
#include <dblib/str_getline.h>
#include <dblib/rfc1123date.h>
#include <dblib/index.h>
#include <dblib/getpeer.h>
#include <dblib/os_dep_file.h>
#include <dbclient/client.h>
#include <dbclient/logstr.h>

#include "deliver.h"

class dbsmtpdt
 {
 public:
  dbsmtpdt(std::istream &_in, std::ostream &_out);
  
  void dbsmtpd();

 protected:
  std::istream &in;
  std::ostream &out;

  std::string command, arg;
  std::string hostname;
  std::string peer, helo;
  std::string maildir;

  bool quit, mail_from_set;
  std::string mail_from;

  struct rcptt
   {
    std::string rcpt;
    std::string user;
   };

  std::list<rcptt> rcpt;

  indext mail_index;

  void help();

  std::string strip_brackets(const std::string &s);
  void check_rcpt(const std::string &address);
  void check_mail(const std::string &address);
  void vrfy(const std::string &address);
  void get_data();
  void deliver(std::list<std::string> &lines);
  void deliver_puffer_erfc(const rfcmessaget &message);
  void deliver_puffer_erfc(const rfcmessaget &message, const std::string &user);

  void get_user(const std::string &address,
                std::string &user,
                std::string &error);

  void get_user_localhost(const std::string &address,
                          std::string &user,
                          std::string &error);

  void process_request();
  void status_response(unsigned code, const std::string &msg);

  void parse_command();

  void read_command();
  void read_header();
  void read_line(std::string &buffer);
 };

/*******************************************************************\

Function: dbsmtpdt::strip_brackets

  Inputs:

 Outputs:

 Purpose:

\*******************************************************************/

std::string dbsmtpdt::strip_brackets(const std::string &s)
 {
  if(!s.empty() && s[0]=='<' && s[s.size()-1]=='>')
    return std::string(s, 1, s.size()-2);

  return s;
 }

/*******************************************************************\

Function: dbsmtpdt::parse_command

  Inputs:

 Outputs:

 Purpose:

\*******************************************************************/

void dbsmtpdt::parse_command()
 {
  if(command=="")
   {
    arg="";
    return;
   }

  std::string::size_type pos=command.find(' ');
  if(pos==std::string::npos)
    return;

  arg=std::string(command, pos+1);
  command=std::string(command, 0, pos);
 }

/*******************************************************************\

Function: dbsmtpdt::check_rcpt

  Inputs:

 Outputs:

 Purpose:

\*******************************************************************/

void dbsmtpdt::check_rcpt(const std::string &address)
 {
  std::string error;

  rcptt r;
  r.rcpt=address;
  get_user(address, r.user, error);

  if(error=="")
   {
    status_response(250, "OK");
    rcpt.push_back(r);
   }
  else
   {
    status_response(550, error);
    logstr << "rcpt rejected: " << address
           << " (" << error << ")";
    logstr.log("check_rcpt");
   }
 }

/*******************************************************************\

Function: dbsmtpdt::vrfy

  Inputs:

 Outputs:

 Purpose:

\*******************************************************************/

void dbsmtpdt::vrfy(const std::string &address)
 {
  std::string error, user;

  get_user(address, user, error);

  if(error=="")
    status_response(252, "OK");
  else
    status_response(550, error);
 }

/*******************************************************************\

Function: dbsmtpd::help

  Inputs:

 Outputs:

 Purpose:

\*******************************************************************/

void dbsmtpdt::help()
 {
  out << "214-The following commands are accepted:\r\n"
         "214-helo noop mail data rcpt help quit rset expn vrfy\r\n"
         "214-\r\n"
         "214 Send bugs reports to: kroening@handshake.de\r\n";
 }

/*******************************************************************\

Function: dbsmtpdt::get_user

  Inputs:

 Outputs:

 Purpose:

\*******************************************************************/

void dbsmtpdt::get_user(const std::string &address,
                        std::string &user,
                        std::string &error)
 {
  error=user="";

  // see if it is the postmaster

  if(address=="postmaster")
   {
    get_user(address+"@"+hostname, user, error);
    return;
   }

  // first a sanity check

  if(address.find(' ')!=std::string::npos)
   {
    error="invalid address";
    return;
   }

  std::string::size_type p=address.find('@');
  if(p==std::string::npos)
   {
    error="invalid address";
    return;
   }

  std::string host(address, p+1);

  // first try specific address

  if(!mail_index.find(address, user))
   {
    if(user=="")
      error="address invalid";
    else if(user=="localhost")
      get_user_localhost(address, user, error);

    return;
   }

  // try host

  if(host=="localhost")
   {
    get_user_localhost(address, user, error);
    return;
   }

  if(!mail_index.find(host, user))
   {
    if(user=="")
      error="host address invalid";
    else if(user=="localhost")
      get_user_localhost(address, user, error);

    return;
   }

  // try domains

  std::string domain=host;

  while(true)
   {
    if(!mail_index.find("."+domain, user))
     {
      if(user=="")
        error="host address invalid";
      else if(user=="localhost")
        get_user_localhost(address, user, error);

      return;
     }

    std::string::size_type s=domain.find('.');
    if(s==std::string::npos) break;
    domain=std::string(domain, s+1);
   }

  error="no service for address";
 }

/*******************************************************************\

Function: dbsmtpdt::get_user_localhost

  Inputs:

 Outputs:

 Purpose:

\*******************************************************************/

void dbsmtpdt::get_user_localhost(const std::string &address,
                                  std::string &user,
                                  std::string &error)
 {
  error=user="";

  // first a sanity check

  if(address.find(' ')!=std::string::npos)
   {
    error="invalid address";
    return;
   }

  std::string::size_type p=address.find('@');
  if(p==std::string::npos)
   {
    error="invalid address";
    return;
   }

  std::string username(address, 0, p);

  if(!mail_index.find(username+"@localhost", user))
   {
    if(user=="") error="user account "+username+" invalid";
    return;
   }

  error="user account "+username+" not found";
 }

/*******************************************************************\

Function: dbsmtpdt::check_mail

  Inputs:

 Outputs:

 Purpose:

\*******************************************************************/

void dbsmtpdt::check_mail(const std::string &address)
 {
  // first a sanity check

  if(address=="")
   {
   }
  else
   {
    if(address.find(' ')!=std::string::npos)
     {
      status_response(553, "no space allowed");
      return;
     }

    std::string::size_type p=address.find('@');
    if(p==std::string::npos)
     {
      status_response(553, "domain part missing");
      return;
     }
   }

  status_response(250, "OK");
  mail_from=address;
  mail_from_set=true;
 }

/*******************************************************************\

Function: dbsmtpdt::dbsmtpdt

  Inputs:

 Outputs:

 Purpose:

\*******************************************************************/

dbsmtpdt::dbsmtpdt(std::istream &_in, std::ostream &_out):
  in(_in), out(_out)
 {
 }

/*******************************************************************\

Function: dbsmtpdt::dbsmtpdt

  Inputs:

 Outputs:

 Purpose:

\*******************************************************************/

void dbsmtpdt::dbsmtpd()
 {
  try
   {  
    hostname=client.config["hostname"];
    maildir=client.config["maildir"];
    std::string maildb=client.config["maildb"];
    peer=getpeer(0);

    if(maildb=="" || maildir=="" || hostname=="")
     {
      status_response(450, "Mail system configuration error");
      return;
     }

    try
     {
      mail_index.open(maildb);
     }

    catch(excpt e)
     {
      status_response(450, "Mail system database unavailable");
      return;
     }

    status_response(220, hostname+" SMTP dbsmtpd");

    mail_from_set=quit=false;

    do
     {
      read_command();

      if(quit)
        break;

      parse_command();

      process_request();
     }
    while(!quit);
   }

  catch(...)
   {
    status_response(450, "internal server error");
   }
 }

/*******************************************************************\

Function: dbsmtpdt::read_command 

  Inputs:

 Outputs:

 Purpose:

\*******************************************************************/

void dbsmtpdt::read_command()
 {
  if(!str_getline_crlf(in, command))
    quit=true;
 }

/*******************************************************************\

Function: dbsmtpdt::process_request

  Inputs:

 Outputs:

 Purpose:

\*******************************************************************/

void dbsmtpdt::process_request()
 {
  upstring(command);

  if(command=="QUIT")
   {
    status_response(221, "Bye");
    quit=true;
   }
  else if(command=="HELO")
   {
    helo=arg;
    status_response(250, hostname);
   }
  else if(command=="HELP")
   {
    help();
   }
  else if(command=="NOOP")
   {
    status_response(250, "OK");
   }
  else if(command=="RCPT")
   {
    if(!mail_from_set)
       status_response(503, "need MAIL command");
    else if(up_string(std::string(arg, 0, 3))=="TO:")
      check_rcpt(dn_string(strip_brackets(std::string(arg, 3))));
    else
      status_response(501, "Syntax: RCPT TO:<address>");
   }
  else if(command=="VRFY")
    vrfy(dn_string(strip_brackets(arg)));
  else if(command=="MAIL")
   {
    if(up_string(std::string(arg, 0, 5))=="FROM:")
      check_mail(strip_brackets(std::string(arg, 5)));
    else
      status_response(501, "Syntax: MAIL FROM:<address>");
   }
  else if(command=="DATA")
   {
    if(rcpt.size()==0)
      status_response(503, "Error: need RCPT");
    else
      get_data();
   }
  else
    status_response(502, "Error: command not implemented");
 }

/*******************************************************************\

Function: dbsmtpdt::status_response

  Inputs:

 Outputs:

 Purpose:

\*******************************************************************/

void dbsmtpdt::status_response(unsigned code, const std::string &msg)
 {
  out << code << " " << msg << "\r\n";
 }

/*******************************************************************\

Function: dbsmtpdt::get_data

  Inputs:

 Outputs:

 Purpose:

\*******************************************************************/

void dbsmtpdt::get_data()
 {
  status_response(354, "End data with <CR><LF>.<CR><LF>");

  std::string line;

  std::list<std::string> lines;

   {
    std::string received;

    received="Received: from ";
    received+=helo;
    received+=" (["+peer+"])";
    lines.push_back(received);
    received="          by "+hostname;
    received+=" (" DBOXVERSIONNR ")";
    received+="; ";
    received+=rfc1123date(time(NULL), 0);
    lines.push_back(received);
   }

  while(true)
   {
    if(!str_getline_crlf(in, line))
     {
      quit=true;
      return;
     }

    if(line==".")
      break;
    else if(line.size()!=0 && line[0]=='.')
      line.erase(0, 1);

    lines.push_back(line);
   }

  deliver(lines);

  rcpt.clear();
  mail_from_set=false;
 }

/*******************************************************************\

Function: dbsmtpdt::deliver_puffer_erfc

  Inputs:

 Outputs:

 Purpose:

\*******************************************************************/

void dbsmtpdt::deliver_puffer_erfc(const rfcmessaget &message,
                                   const std::string &user)
 {
  // found maps file

  std::string puffer_erfc=
    build_path(build_path(maildir, user), user+".erfc");

  std::ofstream out(puffer_erfc.c_str(), 
                    std::ios::out|std::ios::binary|std::ios::app);

  if(!out)
   {
    logstr << "failed to append mail to " << puffer_erfc;
    logstr.log("deliver_puffer_erfc");
   }
  else
   {
    for(std::list<rcptt>::const_iterator it=rcpt.begin();
        it!=rcpt.end(); it++)
      if(it->user==user)
        out << "RCPT: " << it->rcpt << "\r\n";

    out << "FROM: " << mail_from << "\r\n";

    out << "LEN: " << message.len+message.headerlen << "\r\n";
    out << "\r\n";
    message.copy(out);
   }
 }

/*******************************************************************\

Function: dbsmtpdt::deliver_puffer_erfc

  Inputs:

 Outputs:

 Purpose:

\*******************************************************************/

void dbsmtpdt::deliver_puffer_erfc(const rfcmessaget &message)
 {
  // get set of users

  std::set<std::string> user_set;

  for(std::list<rcptt>::const_iterator it=rcpt.begin();
      it!=rcpt.end(); it++)
    user_set.insert(it->user);

  // deliver to set of users with maps file

  for(std::set<std::string>::const_iterator it=user_set.begin();
      it!=user_set.end(); it++)
   {
    if(access(build_path(build_path(maildir, *it), "maps").c_str(), 00)==0)
      deliver_puffer_erfc(message, *it);
   }
 }

/*******************************************************************\

Function: dbsmtpdt::deliver

  Inputs:

 Outputs:

 Purpose:

\*******************************************************************/

void dbsmtpdt::deliver(std::list<std::string> &lines)
 {
  // precompute size to prevent reallocation

  std::string::size_type s=0;

  for(std::list<std::string>::const_iterator it=lines.begin();
      it!=lines.end(); it++)
    s+=it->size()+2;

  std::string message_text;

  message_text.reserve(s);

  // add up lines

  for(std::list<std::string>::const_iterator it=lines.begin();
      it!=lines.end(); it++)
   {
    message_text+=*it;
    message_text+="\r\n";
   }

  // build message

  rfcmessaget message;
  message.data=message_text.c_str();
  message.endoffile=message.endofmail=message.data+message_text.size();

  // parse header
  message.parse();

   {
    std::string header;

    // log the message
    logstr << "from=<" << mail_from << ">";
    logstr << ", size=" << message_text.size();

    if(message.findsingleheader(header, "Message-ID"))
      logstr << ", msgid=" << header;

    logstr << ", relay=" << peer;

    logstr.log("deliver");
   }

  // deliver message

  try
   {
    for(std::list<rcptt>::const_iterator it=rcpt.begin();
        it!=rcpt.end(); it++)
     {
      if(it->user.empty())
        throw excpt("internal error: empty user (1)");

      ::deliver(message, it->user, it->rcpt, mail_from);

      // log it
      logstr << "to=<" << it->rcpt << ">";
      logstr << ", user=" << it->user;
      logstr.log("deliver");
     }

    status_response(250, "OK");

    // deliver to puffer.erfc - if required
    deliver_puffer_erfc(message);
   }

  catch(excpt e)
   {
    status_response(450, e.what());
    logstr << "delivery failed: " << e.what();
    logstr.log("deliver");
   }
 }

/*******************************************************************\

Function: dbsmtpd

  Inputs:

 Outputs:

 Purpose:

\*******************************************************************/

void dbsmtpd()
 {
  dbsmtpdt dbsmtpd(std::cin, std::cout);

  dbsmtpd.dbsmtpd();
 }
