#include <stdio.h>
#include <string.h>
#include <fcntl.h>
#include <sys/stat.h>
#include <stdlib.h>
#include <unistd.h>
#include <time.h>
#include <ctype.h>

#include <dbox.h>
#include <tools.h>
#include <log_serv.h>
#include <folder.h>
#include <folderindex.h>
#include <dboxpath.h>
#include <mtoolzc.h>
#include <sfolderzc.h>
#include <zconnect.h>
#include <stringtools.h>
#include <dblib/rfc1522.h>

static void makeexternfilename(
  std::string &filename,
  const zcmessaget &m,
  const std::string &directory)
 {
  const char *start, *file;
  char tempstr[DIRLEN];

  start=m.data;
  if(m.findheader(&start, "FILE", &file))
    m.copyheadertext(tempstr, m.bet, DIRLEN-1);
  else
    m.copyheadertext(tempstr, file, DIRLEN-1);

  makeexternfilename(filename, tempstr, directory);
 }

static bool checkfolderdir(const char *directory)
 {
  char tempstr[DIRLEN];

  snprintf(tempstr, DIRLEN, "%s.", directory);
  if(access(tempstr, 00)==0) return FALSE;

  return TRUE;

  #if 0
  strmaxcpy(tempstr, directory, DIRLEN-1);
  tptr=strchr(tempstr, DIRCHAR[0])+1;

  do
   {
    tptr=strchr(tptr, DIRCHAR[0]);
    if(tptr!=NULL) *tptr=0;
    error=mkdir(tempstr, 0700)!=0;
    if(tptr!=NULL) *tptr=DIRCHAR[0];
    tptr++;
   }
  while(tptr!=NULL && *tptr!=0);
  return error;
  #endif
 }

static bool append_fi(const folderindext &fi, folderpt &fp)
 {
  char filename[DIRLEN];

  snprintf(filename, DIRLEN, "%sindex", fp.getpath());

  int handle=open(filename, O_WRONLY|O_APPEND|O_CREAT|O_BINARY,
    S_IREAD|S_IWRITE);

  if(handle<0)
   {
    log_serv.printf("append_fi",
      "fatal: failed to open index file \"%s\"", filename);
    return TRUE;
   }

  if(write(handle, &fi, sizeof fi)!=sizeof fi)
   {
    log_serv.printf("append_fi",
      "fatal: error while writing to index file \"%s\"", filename);
    close(handle);
    return TRUE;
   }

  close(handle);
  return FALSE;
 }

bool spoolinfolder(const zcmessaget &m, const char *folder, 
                   bool forceextern,
                   const char *envelope_to,
                   const char *envelope_from)
 {
  bool mail;
  folderindext fi;
  FILE *outfile;
  char tempstr[DIRLEN],
       envelope_to_header[TEXTLEN*2],
       envelope_from_header[TEXTLEN*2];
  //int handle;

  if(envelope_to!=NULL && envelope_to[0]!=0)
    snprintf(envelope_to_header, TEXTLEN*2,
      "X-POP3-Rcpt:\t%s\r\n"
      "Envelope-To:\t%s\r\n", 
      envelope_to, envelope_to);
  else
    envelope_to_header[0]=0;

  if(envelope_from!=NULL && envelope_from[0]!=0)
    snprintf(envelope_from_header, TEXTLEN*2, "X-Envelope-From:\t%s\r\n", 
      envelope_from);
  else
    envelope_from_header[0]=0;

  mail=folder[0]!='/';

  folderpt fp(folder);
  fp.setfilename();

  if(m.abs==NULL || m.bet==NULL || m.eda==NULL)
    return TRUE;

  if(checkfolderdir(fp.getpath()))
   {
    log_serv.printf("spoolinfolder",
      "fatal: failed to access folder directory \"%s\"", fp.getpath());
    return TRUE;
   }

  if(fp.lock("spoolinfolder"))
   {
    log_serv.printf("spoolinfolder",
      "Fatal: Folder %s is locked", fp.name);
    return TRUE;
   }

  fp.read();

  snprintf(tempstr, DIRLEN, "%sheader", fp.getpath());
  outfile=fopen(tempstr, "ab");
  if(outfile==NULL)
   {
    log_serv.printf("spoolinfolder",
      "Fatal: Zugriff auf \"%s\" nicht mglich", tempstr);
    fp.unlock();
    return TRUE;
   }

  fi.clear();

   {
    char bet[TEXTLEN];
    m.copyheadertext(bet, m.bet, TEXTLEN-1);
    std::string new_bet=bet;
    ibm2iso(new_bet);
    new_bet=decode_rfc1522(new_bet);
    iso2ibm(new_bet);
    strmaxcpy(fi.bet, new_bet.c_str(), TEXTLEN-1);
   }

  m.copyheadertext(fi.abs, m.abs, TEXTLEN-1);
  m.copyheadertext(fi.mid, m.mid, TEXTLEN-1);
  fi.eda=parseeda(m.eda);
  time(&fi.received);
  fi.len=m.len;
  fi.headerlen=m.headerlen+
               strlen(envelope_to_header)+strlen(envelope_from_header);
  if(forceextern || fi.len>EXTERNLEN) fi.flags.bodyextern=TRUE;
  fi.flags.binary=(m.typ!=NULL);
  fi.headeroffs=filelength(fileno(outfile));
  fi.id=atol(fp|"nextid");
  if(fi.id==0) fi.id=1;

  #if 0
   {
    char *dbox_rfc, *start;
    start=m.data;
    if(!m.findheader(&start, "_DBOX-RFC", &dbox_rfc))
      fi.flags.rfc=TRUE;
   }
  #endif

  fprintf(outfile, "%s", envelope_from_header);
  fprintf(outfile, "%s", envelope_to_header);

  if(fi.flags.bodyextern)
   {
    FILE *outfile2;

    if(m.copyheader(outfile))
     {
      log_serv.printf("spoolinfolder",
        "fatal: failed to copy header into \"%s\"", tempstr);
      fclose(outfile);
      fp.unlock();
      return TRUE;
     }

    std::string externfilename;
    makeexternfilename(externfilename, m, fp.getpath());

    outfile2=fopen(externfilename.c_str(), "wb");
    if(outfile2==NULL)
     {
      log_serv.printf("spoolinfolder",
        "fatal: failed to open \"%s\" for writing",
        externfilename.c_str());
      fclose(outfile);
      fp.unlock();
      return TRUE;
     }

    strmaxcpy(fi.file, strrchr(externfilename.c_str(), DIRCHAR[0])+1,
              TEXTLEN-1);

    if(m.copybody(outfile2))
     {
      log_serv.printf("spoolinfolder",
        "fatal: failed to copy body into \"%s\"",
        externfilename.c_str());
      fclose(outfile);
      fclose(outfile2);
      fp.unlock();
      return TRUE;
     }

    fclose(outfile2);
   }
  else
   {
    if(m.copy(outfile))
     {
      log_serv.printf("spoolinfolder",
        "fatal: failed to copy message into \"%s\"", tempstr);
      fclose(outfile);
      fp.unlock();
      return TRUE;
     }
   }

  fclose(outfile);

  fp.set("nextid", fi.id+1);
  fp.write();

  if(append_fi(fi, fp))
   {
    fp.unlock();
    return TRUE;
   }

  fp.unlock();

  return FALSE;
 }

bool spool_in_folder(const zcmessaget &m,
                     const std::string &folder, 
                     bool forceextern,
                     const std::string &envelope_to,
                     const std::string &envelope_from)
 {
  return spoolinfolder(m, folder.c_str(), forceextern,
                       envelope_to.c_str(),
                       envelope_from.c_str());
 }
