#ifndef DBOX_HTMLFORM_H

#define DBOX_HTMLFORM_H

#include <libdbox.h>
#include <vector>
#include <string>
#include <iostream>

#define MAX_LINES 10000

class htmlchart
 {
 public:
  bool ibm_charset, islink, bold, underline, italic, preformat, center;
  bool marked;
  char ch;
  unsigned link, form;
    
  void clear();
  htmlchart() { clear(); }
  htmlchart(char _ch) { clear(); ch=_ch; }
 };
 
struct tag_paramt
 {
  std::string id, value;
 };

const std::string &search(const std::vector<tag_paramt> &params, const std::string &id);
bool search(const std::vector<tag_paramt> &params, const std::string &id, 
            std::string &value);

class htmlformatt
 {
 public:
  std::string title; // ISO
 
  enum alignmentt { LEFT, CENTER, RIGHT, BLOCK }
    default_alignment, current_alignment;
    
  struct form_inputt
   {
    enum form_typet { HIDDEN, DEFAULT, SUBMIT } type;
    std::string name, value;
    unsigned size, posx, posy, scrollx, scrolly, sizex, sizey;
    unsigned form;
   };

  struct formt
   {
    std::string action, method;
   };
  
  class linkt
   {
   public:
    std::string href;
    
    bool is_form;
    unsigned form_input;
    
    linkt() { is_form=FALSE; form_input=0; }
   };
   
  std::vector<formt> forms;
  std::vector<form_inputt> form_inputs;
  std::vector<linkt> links;
 
  std::vector<std::vector <htmlchart> > lines;
  
  void format(std::istream &in);
  void set_width(unsigned _width) { width=_width; }
 
  htmlformatt();
  
 protected:
  unsigned width, line;
  unsigned indent, max_width;
  std::vector<int> indent_stack;
  bool attr_bold, attr_underline, attr_title, attr_link,
       attr_table, attr_italic, attr_preformat, attr_ibm_charset,
       attr_center, attr_script, do_get_max_width;
  std::vector <htmlchart> word;

  void gen_htmlchar(htmlchart &htmlchar);
  void new_line();
  void line_break();
  void flush_word();
  void tag_params(std::istream &in, std::vector<tag_paramt> &params);
  void do_tag(std::istream &in, const std::string &name, const std::vector<tag_paramt> &params);
  void get_tag(std::istream &in, std::string &name, std::vector<tag_paramt> &params);
  void do_entity(const std::string &name);
  unsigned get_max_width(std::istream &in);
  void tag(std::istream &in);
  void entity(std::istream &in);
  void html_comment(std::istream &in);
  void text(char ch);
  void text(const std::string &str);
  void entity_text(const std::string &str);
  
  // table support
  class tablet
   {
   public:
    bool border;
    unsigned columns;
    void clear();
    void set_params(const std::vector<tag_paramt> &params);
    
    struct columnt
     {
      std::string html;
      unsigned max_width, width, col_span;
      htmlformatt *hf;
      unsigned link_offset;
      alignmentt align;
      bool th;
     };
     
    class rowt
     {
     public:
      std::vector<columnt> columns;
      unsigned height;
      rowt() { height=0; }
     };
     
    std::vector<rowt> rows;
   } table;
 
  void render_table();
  void do_td(std::istream &in, const std::vector<tag_paramt> &params, bool th);
  void do_tr(const std::vector <tag_paramt> &params);
  void draw_table_border(bool upper);
  void draw_table_separator(unsigned row_nr);
  void draw_td(tablet::columnt &column, unsigned l);
  void td_merge(tablet::columnt &column);
  void form(const std::vector<tag_paramt> &params);
  void form_input(const std::vector<tag_paramt> &params);
 };
 
#endif
