/*

  DBOX Module
  Copyright (C) 1994-2000 Daniel Kroening <kroening@handshake.de>

  Purpose: 

*/

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <dirent.h>
#include <fcntl.h>
#include <time.h>
#include <sys/stat.h>

#include <fstream>
#include <string>

#include <dboxpath.h>
#include <nls.h>
#include <ui.h>
#include <dblib/rfc1123date.h>
#include <tools.h>
#include <log_serv.h>
#include "online.h"
#include "gen_from.h"
#include "help.h"
#include "writemail.h"
#include "editor.h"

void writemail2(std::string &to, std::string &subject,
                const std::string &in_reply_to, std::istream *quotefile)
 {
  #ifndef DBOX_STANDALONE
  if(strchr(to.c_str(), '@')==NULL)
   {
    to+="@";
    to+=config.boxname;
    to+="."; 
    to+=config.domain;
   }
  #endif

  std::string filename;
  //snprintf(filename, DIRLEN, DBOXPORTTEMPDIR "mail.tmp", 
  //  config.dboxdir, config.port);

   {
    std::ofstream outfile(filename.c_str(), std::ios::out|std::ios::trunc|std::ios::binary);
    if(!outfile) return;

    //if(quotefile!=NULL)
    //  filecopy(quotefile, outfile, filelength(fileno(quotefile)));

    //snprintf(tempstr, TEXTLEN*2, BOXUSERDIR "sig", 
    //  config.boxdir, status.username);

    //infile=fopen(tempstr, "rb");
    //if(infile!=NULL)
    // {
    //  fprintf(outfile, "\r\n");
    //  filecopy(infile, outfile, filelength(fileno(infile)));
    //  fclose(infile);
    // }
   }

  std::string title="Nachricht an"+to;
  if(doedit(filename.c_str(), title.c_str()))
   {
    if(status.dologout) return;
    info_box(online, "Nachricht nicht abgeschickt.");
   }
  else
   {
    if(status.dologout) return;

    std::string reply_to;

    online.setattr(2, FALSE);
    online.box(online.sizex/2-30, 8, online.sizex/2+30, 19,
             BOXFRAMEDBL, nls["online.reallysend"], TRUE);

    online.setcursor(online.sizex/2-18, 10);
    online.printf("Empfnger:");
    online.setcursor(online.sizex/2-18, 12);
    online.printf("Betreff:");

    online.setcursor(online.sizex/2-18, 14);
    online.printf("Antwort an:");

    online.setcursor(online.sizex/2-18, 15);
    online.printf("Empfangsbesttigung:");

     {
      online.setcursor(33, 10);
      online.setattr(2, TRUE);
      online.printf("%.26s", to.c_str());
      online.setattr(2, FALSE);
     }

    menut menu(&online);

    editstringmet subject_me;
    subject_me.x=33;
    subject_me.y=12;
    subject_me.sizex=24;
    subject_me.maxlen=60;
    subject_me.text=&subject;
    menu.add(&subject_me);

    editstringmet reply_to_me;
    reply_to_me.x=33;
    reply_to_me.y=14;
    reply_to_me.sizex=24;
    reply_to_me.maxlen=60;
    reply_to_me.text=&reply_to;
    menu.add(&reply_to_me);

    checkboxmet return_receipt_me;
    return_receipt_me.x=33;
    return_receipt_me.y=15;
    menu.add(&return_receipt_me);

    ok_buttonmet ok_me;
    ok_me.x=online.sizex/2-14;
    ok_me.y=17;
    ok_me.id=5;
    menu.add(&ok_me);

    abort_buttonmet abort_me;
    abort_me.x=online.sizex/2+3;
    abort_me.y=17;
    abort_me.id=6;
    menu.add(&abort_me);

    switch(menu.menu())
     {
     case 5:
      std::string from, org;

      gen_from(status, status.username, from, org, TRUE);

      std::ifstream infile(filename.c_str(), std::ios::in|std::ios::binary);

      std::string outfilename;
      getdboxporttempdir(outfilename, config.dboxdir, config.port);
      outfilename+="msg.tmp";

      std::ofstream outfile(outfilename.c_str(), std::ios::out|std::ios::trunc);
      if(!outfile)
       {
        log_serv.printf("writemail2", "Failed to create \"%s\"", 
          outfilename.c_str());
        return;
       }

      outfile << "From: " << from << "\r\n";

      if(reply_to!="")
        outfile << "Reply-To: " << reply_to << "\r\n";

      outfile << "Subject: " << subject << "\r\n";

      if(in_reply_to!="")
        outfile << "In-Reply-To: " << in_reply_to << "\r\n";

      outfile << "Date: " << rfc1123date(time(NULL), 0) << "\r\n";

      outfile << "To: " << to << "\r\n";
      outfile << "X-Mailer: DBOX mailer\r\n";
      
      if(return_receipt_me.checked)
        outfile << "Request-Return-Receipt:\r\n";

      outfile << "Message-Id: mailer." << (status.midcounter++)
              << "." << time(NULL) << config.boxname << "." 
              << config.domain << "\r\n";

      if(org!="")
        outfile << "Organization: " << org << "\r\n";

      outfile << "\r\n";

      //filecopy(infile, outfile, len);

      //remove(filename);

      //snprintf(filename, DIRLEN, BOXSPOOLINZCMAILDIR "%s.%lu-%lu-%u",
      //  config.boxdir, news?config.boxname:status.username,
      //  status.midcounter++, time(NULL), config.port);

      //rename(outfilename, filename);

      info_box(online, "Nachricht verschickt!");
     }
   }
 }

void writemail(const std::string &_to, const std::string &_subject,
               const std::string &in_reply_to, std::istream *quotefile)
 {
  std::string subject=_subject, to=_to;

  while(TRUE)
   {
    online.setattr(2, FALSE);
    online.box(online.sizex/2-30, 10, online.sizex/2+30, 18,
               BOXFRAMEDBL, "EMAIL", TRUE);

    online.setcursor(online.sizex/2-28, 12);
    online.printf("Empfnger:");
    online.setcursor(online.sizex/2-28, 14);
    online.printf("Betreff:");

    menut menu(&online);
    editstringmet to_me;

    if(_to!="")
     {
      to_me.x=23;
      to_me.y=12;
      to_me.sizex=44;
      to_me.maxlen=60;
      to_me.text=&to;
      menu.add(&to_me);
     }
    else
     {
      online.setattr(2, TRUE);
      online.setcursor(online.sizex/2-17, 12);
      online.printf("%.44s", to.c_str());
      online.setattr(2, FALSE);
     }

    editstringmet subject_me;
    subject_me.x=23;
    subject_me.y=14;
    subject_me.sizex=44;
    subject_me.maxlen=TEXTLEN-1;
    subject_me.text=&subject;
    menu.add(&subject_me);

    ok_buttonmet ok_me;
    ok_me.id=2;
    ok_me.x=22;
    ok_me.y=16;
    menu.add(&ok_me);

    abort_buttonmet abort_me;
    abort_me.id=3;
    abort_me.x=31;
    abort_me.y=16;
    menu.add(&abort_me);

    switch(menu.menu())
     {
     case 2:
      writemail2(to, subject, in_reply_to, quotefile);
      break;

     case MENU_HELP:
      online.clrscr();
      if(status.dologout) return;
      break;

     default:
      return;
     }
   }
 }
