/*

  DBOX Module
  Copyright (C) 1994-2000 Daniel Kroening <kroening@handshake.de>

  Purpose: 

*/

#include <sstream>
#include <fstream>
#include <iomanip>

#include <nls.h>
#include <stringtools.h>
#include <account.h>
#include <log_serv.h>

#include "challenge.h"
#include "htmlforms.h"

void htmlformst::processmacro(std::ostream &out,
                              const std::string &expression, 
                              const std::string &type)
 {
  if(expression=="MONTH")
   {
    std::ostringstream temp;
    temp << "month" << (tm.tm_mon+1);
    out << (nls.get(temp.str()));
   }
  else if(expression=="DATE")
   {
    out << std::setw(2) << std::setfill('0') << tm.tm_mday << '.'; 
    out << std::setw(2) << std::setfill('0') << tm.tm_mon+1 << '.';
    out << std::setw(2) << std::setfill('0') << tm.tm_year+1900;
   }
  else if(expression=="QUARTER")
   {
    out << (tm.tm_mon/3)+1;
   }
  else if(stringcasecmp(expression, "WEBONLINE.BGCOLOR")==0)
    out << config.webonline_bgcolor;
  else if(stringcasecmp(expression, "WEBONLINE.THBGCOLOR")==0)
    out << config.webonline_thbgcolor;
  else if(stringcasecmp(expression, "WEBONLINE.THFCOLOR")==0)
    out << config.webonline_thfcolor;
  else if(stringcasecmp(expression, "WEBONLINE.TABLEBGCOLOR1")==0)
    out << config.webonline_tablebgcolor1;
  else if(stringcasecmp(expression, "WEBONLINE.TABLEBGCOLOR2")==0)
    out << config.webonline_tablebgcolor2;
  else
   {
    for(const_iterator it=begin(); it!=end(); it++)
      if(stringcasecmp(it->first, expression)==0)
       {
        out << it->second;
        return;
       }

    out << "UNKNOWN: \"" << expression << "\"";
   }
 }

htmlformst::~htmlformst()
 {
 }

htmlformst::htmlformst()
 {
  time_t now;
  time(&now);
  tm=*localtime(&now);
 }

void htmlformst::processmacro(std::ostream &out,
                              const std::string &macro)
 {
  std::string expression, type;
  unsigned i;

  for(i=0; i<macro.size() && macro[i]!=':'; i++)
    expression+=macro[i];

  if(i<macro.size())
    for(i++; i<macro.size(); i++)
      type+=macro[i];
    
  processmacro(out, expression, type);
 }

void htmlformst::serve(const std::string &in, std::ostream &out)
 {
  std::ifstream infile(in.c_str());

  if(infile)
    serve(infile, out);
  else
    log_serv.printf("htmlformst::serve", "file \"%s\" not found", 
      in.c_str());
 }

void htmlformst::serve(std::istream &in, std::ostream &out)
 {
  char ch;

  while(in.read(&ch, 1))
   {
    if(ch=='<')
     {
      char ch2;
      if(in.read(&ch2, 1))
       {
        if(ch2=='$')
         {
          std::string expression;

          while(in.read(&ch, 1))
           {
            if(ch=='>')
              break;
            else
              expression+=ch;
           }

          processmacro(out, expression);

          continue;
         }

        out << ch << ch2;
        continue;
       }
     }

    out << ch;
   }
 }

void htmlpformst::processmacro(std::ostream &out, 
                               const std::string &expression,
                               const std::string &type)
 {
  if(strnicmp(expression.c_str(), "PROFILE.", 8)!=0)
   {
    htmlformst::processmacro(out, expression, type);
    return;
   }
  
  std::string data;

  p->get(expression.c_str()+8, data);
  ibm2iso(data);

  if(type=="")
    out << data;
  else if(stringcasecmp(type, "GROUP.SELECT")==0)
   {
    for(unsigned i=0; i<=p->pconfig->groupdb.size(); i++)
     {
      out << "<option value=\"";

      const char *value;
      if(i<p->pconfig->groupdb.size())
        value="";
      else
        value=p->pconfig->groupdb[i];

      out << value << "\"";

      if(stringcasecmp(data, value)==0)
        out << " checked";

      out << ">" << value << "</option>\n";
     }
   }
  else if(stringcasecmp(type, "BOOL.CHECKED")==0)
   {
    if(atoi(data.c_str())!=0) out << "CHECKED";
   }
  else if(stringcasecmp(type, "BOOL.NOYES")==0)
   {
    out << nls.get(atoi(data.c_str())?"misc.yes":"misc.no");
   }
  else if(stringcasecmp(type, "CURRENCY")==0)
   {
    amountt amount;
    char tempstr[TEXTLEN];
    amount.amount=atol(data.c_str());
    snprintf(tempstr, TEXTLEN, "%s.scale", expression.c_str()+8);
    p->get(tempstr, data);
    amount.scale=atol(data.c_str());
    currency_string(tempstr, amount, TRUE, SYMBOL_HTML);
    out << tempstr;
   }
  else
    out << "Type \"" << type << "\" unknown";
 }

void htmlpformst::setprofile(pt *_p)
 {
  p=_p;
 }
  
void htmlupformst::setprofile(userpt *_p)
 {
  p=_p;
  up=_p;
 }

void htmlupformst::processmacro(std::ostream &out, 
                                const std::string &expression,
                                const std::string &type)
 {
  if(stringcasecmp(expression, "ALIASLIST")==0)
   {
    processmacro_aliaslist(out, type);
    return;
   }

  if(stringcasecmp(expression, "CHALLENGE")==0)
   {
    std::string challenge;
    get_challenge(challenge, *up);
    out << challenge;
    return;
   }

  return htmlpformst::processmacro(out, expression, type);
 }
  
void htmlupformst::processmacro_aliaslist(std::ostream &out,
                                          const std::string &type)
 {
  if(stringcasecmp(type, "SELECT")==0)
   {
    out << "<option value=\"" << p->name << "\" checked>"
        << p->name << "</option>\n";

    char filename[DIRLEN];
    snprintf(filename, DIRLEN, "%salias", up->getpath());
    std::ifstream infile(filename);

    if(infile)
     {
      char line[1000];

      while(infile.get(line, 900))
       {
        char ch, *tptr;
        infile.get(ch); // \n lesen
        
        tptr=strchr(line, '\n'); if(tptr!=NULL) *tptr=0;
        tptr=strchr(line, '\r'); if(tptr!=NULL) *tptr=0;

        out << "<option value=\"" << line << "\">" 
            << line << "</option>\n";
       }
     }
   }
  else
    out << "Type \"" << type << "\" unknown";
 }


